// -------------------------------------------------------------
// 3d code by Georgy Moshkin (Delphi)
// licence: public domain
// -------------------------------------------------------------
unit mymatrix;

interface

uses myvectors;

type TMatrix = array[0..3,0..3] of single;

function matrix_rotate(axis: TVector;angle: Single): TMatrix;

function Mat(a00,a01,a02,a03,
             a10,a11,a12,a13,
             a20,a21,a22,a23,
             a30,a31,a32,a33: single):TMatrix;

function RotMatX(A:single):TMatrix;
function RotMatY(A:single):TMatrix;
function RotMatZ(A:single):TMatrix;
function TraMat(x,y,z:single):TMatrix;
function ConcatMatrix(const Ma: TMatrix; const Mb: TMatrix):TMatrix;
Procedure ApplyMatrix(Var Vert: TVector; const M: TMatrix);
Procedure InvApplyMatrix(Var Vert: TVector; const M: TMatrix);

implementation

function Mat(a00,a01,a02,a03,
             a10,a11,a12,a13,
             a20,a21,a22,a23,
             a30,a31,a32,a33: single):TMatrix;
begin

result[0,0]:=a00;
result[0,1]:=a01;
result[0,2]:=a02;
result[0,3]:=a03;
result[1,0]:=a10;
result[1,1]:=a11;
result[1,2]:=a12;
result[1,3]:=a13;
result[2,0]:=a20;
result[2,1]:=a21;
result[2,2]:=a22;
result[2,3]:=a23;
result[3,0]:=a30;
result[3,1]:=a31;
result[3,2]:=a32;
result[3,3]:=a33;
end;

function RotMatX(A:single):TMatrix;
var cosA,sinA:single;
begin
cosA:=cos(A);
sinA:=sin(A);
result:= mat(1, 0,    0,     0,
             0, cosA, -sinA, 0,
             0, sinA,  cosA, 0,
             0, 0,     0,    1);
end;

function RotMatY(A:single):TMatrix;
var cosA,sinA:single;
begin
cosA:=cos(A);
sinA:=sin(A);
result:= mat(cosA, 0, sinA, 0,
	     0,    1,  0,    0,
	     -sinA,0,  cosA, 0,
	     0,    0,  0,    1);
end;

function RotMatZ(A:single):TMatrix;
var cosA,sinA:single;
begin
cosA:=cos(A);
sinA:=sin(A);
result:= mat(cosA, -sinA, 0, 0,
	     sinA,  cosA, 0, 0,
	     0,     0,    1, 0,
	     0,     0,    0, 1);

end;

function TraMat(x,y,z:single):TMatrix;
begin
result:=mat(1,0,0,x,
            0,1,0,y,
            0,0,1,z,
            0,0,0,1);
end;

function ConcatMatrix(const Ma: TMatrix; const Mb: TMatrix):TMatrix;
Var row,clm: integer;
begin
  for row := 0 to 3 do
   for clm := 0 to 3 do
    Result[row,clm] := ma[row,0] * mb[0,clm] +
                       ma[row,1] * mb[1,clm] +
                       ma[row,2] * mb[2,clm] +
                       ma[row,3] * mb[3,clm];
end;

Procedure ApplyMatrix(Var Vert: TVector; const M: TMatrix);
Var N: TVector;
begin
  N.x := vert.x * M[0,0] + Vert.y * M[0,1] + Vert.z * M[0,2] + M[0,3];
  N.y := vert.x * M[1,0] + Vert.y * M[1,1] + Vert.z * M[1,2] + M[1,3];
  N.z := vert.x * M[2,0] + Vert.y * M[2,1] + Vert.z * M[2,2] + M[2,3];
  Vert:=N;
end;


Procedure InvApplyMatrix(Var Vert: TVector; const M: TMatrix);
Var N: TVector;
begin
  vert.x := vert.x -  M[0,3];
  vert.y := vert.y -  M[1,3];
  vert.z := vert.z -  M[2,3];
  N.x := vert.x * M[0,0] + Vert.y * M[1,0] + Vert.z * M[2,0];
  N.y := vert.x * M[0,1] + Vert.y * M[1,1] + Vert.z * M[2,1];
  N.z:=  vert.x * M[0,2] + Vert.y * M[1,2] + Vert.z * M[2,2];
  vert:=N;
end;


function matrix_rotate(axis: TVector;angle: Single): TMatrix;
var
  m: TMatrix;
  cosA, sinA: Single;
begin

  // Return a rotation matrix for an arbitrary axis through the origin.
  m := TraMat(0,0,0);
  cosA := cos(angle);
  sinA := sin(angle);

  m[0,0] := cosA + (1 - cosA)*axis.x*axis.x;
  m[1,0] := (1 - cosA)*axis.x*axis.y - axis.z*sinA;
  m[2,0] := (1 - cosA)*axis.x*axis.z + axis.y*sinA;
  m[0,1] := (1 - cosA)*axis.x*axis.z + axis.z*sinA;
  m[1,1] := cosA + (1 - cosA)*axis.y*axis.y;
  m[2,1] := (1 - cosA)*axis.y*axis.z - axis.x*sinA;
  m[0,2] := (1 - cosA)*axis.x*axis.z - axis.y*sinA;
  m[1,2] := (1 - cosA)*axis.y*axis.z + axis.x*sinA;
  m[2,2] := cosA + (1 - cosA)*axis.z*axis.z;

  Result := m;

end;




end.
